@@ -1,8 +1,9 @@
+# -*- coding: utf-8 -*-
+
 from gi.repository import Peas
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import Gio
-from gi.repository import GConf
 from gi.repository import Pango
 from gi.repository import Totem
 import gobject
@@ -243,7 +244,7 @@ class OpenSubtitlesModel(object):
             if self.token:
                 return True
 
-        self.message = _('Could not contact the OpenSubtitles website')
+        self.message = _(u'Could not contact the OpenSubtitles website')
 
         return False
 
@@ -259,12 +260,12 @@ class OpenSubtitlesModel(object):
             try:
                 result = self.server.SearchSubtitles(self.token, [searchdata])
             except xmlrpclib.ProtocolError:
-                self.message = _('Could not contact the OpenSubtitles website')
+                self.message = _(u'Could not contact the OpenSubtitles website')
 
             if result.get('data'):
                 return result['data']
             else:
-                self.message = _('No results found')
+                self.message = _(u'No results found')
 
         return None
 
@@ -276,13 +277,13 @@ class OpenSubtitlesModel(object):
             try:
                 result = self.server.DownloadSubtitles(self.token, [subtitleId])
             except xmlrpclib.ProtocolError:
-                self.message = _('Could not contact the OpenSubtitles website')
+                self.message = _(u'Could not contact the OpenSubtitles website')
 
             if result and result.get('status') == OK200:
                 try:
                     subtitle64 = result['data'][0]['data']
                 except:
-                    self.message = _('Could not contact the OpenSubtitles website')
+                    self.message = _(u'Could not contact the OpenSubtitles website')
                     return None
 
                 import StringIO, gzip, base64
@@ -297,24 +298,22 @@ class OpenSubtitlesModel(object):
 class OpenSubtitles(gobject.GObject, Peas.Activatable):
     __gtype_name__ = 'OpenSubtitles'
 
+    object = gobject.property(type = gobject.GObject)
+
     def __init__(self):
         self.dialog = None
-        self.gconf_client = GConf.Client.get_default()
-        self.GCONF_BASE_DIR = "/apps/totem/plugins/opensubtitles/"
-        self.GCONF_LANGUAGE = "language"
+        self.totem = None
+        self.settings = Gio.Settings.new ('org.gnome.totem.plugins.opensubtitles')
 
     # totem.Plugin methods
 
-    def do_activate(self, totem_object):
+    def do_activate(self):
         """
         Called when the plugin is activated.
         Here the sidebar page is initialized(set up the treeview, connect 
         the callbacks, ...) and added to totem.
-
-        @param totem_object:
-        @type  totem_object: {totem.TotemObject}
         """
-        self.totem = totem_object
+        self.totem = self.object
 	self.filename = None
 
         self.manager = self.totem.get_ui_manager()
@@ -327,7 +326,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         server = xmlrpclib.Server('http://api.opensubtitles.org/xml-rpc')
         self.model = OpenSubtitlesModel(server)
 
-    def do_deactivate(self, totem):
+    def do_deactivate(self):
         if self.dialog:
             self.dialog.destroy()
 	    self.dialog = None
@@ -336,8 +335,8 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
     # UI related code
 
-    def os_build_dialog(self, action, totem_object):
-        builder = Totem.plugin_load_interface ("opensubtitles", "opensubtitles.ui", True, totem_object.get_main_window (), self)
+    def os_build_dialog(self, action):
+        builder = Totem.plugin_load_interface ("opensubtitles", "opensubtitles.ui", True, self.totem.get_main_window (), self)
 
         # Obtain all the widgets we need to initialize
         combobox =       builder.get_object('language_combobox')
@@ -352,36 +351,40 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         # Set up and populate the languages combobox
         renderer = Gtk.CellRendererText()
-        sorted_languages = Gtk.TreeModelSort.new_with_model(languages)
+        sorted_languages = Gtk.TreeModelSort (model = languages)
         sorted_languages.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         combobox.set_model(sorted_languages)
         combobox.pack_start(renderer, True)
         combobox.add_attribute(renderer, 'text', 0)
 
-        self.model.lang = self.gconf_get_str (self.GCONF_BASE_DIR + self.GCONF_LANGUAGE, self.model.lang)
+        lang = self.settings.get_string ('language')
+        if lang is not None:
+            self.model.lang = lang
+
         for lang in LANGUAGES_STR:
             it = languages.append(lang)
             if LANGUAGES[lang[1]] == self.model.lang:
-                parentit = sorted_languages.convert_child_iter_to_iter (it)
-                combobox.set_active_iter(parentit)
+                (success, parentit) = sorted_languages.convert_child_iter_to_iter (it)
+                if success:
+                    combobox.set_active_iter (parentit)
 
         # Set up the results treeview 
         renderer = Gtk.CellRendererText()
         self.treeview.set_model(self.liststore)
         self.treeview.set_headers_visible(False)
         renderer.set_property('ellipsize', Pango.EllipsizeMode.END)
-        column = Gtk.TreeViewColumn(_("Subtitles"), renderer, text=0)
+        column = Gtk.TreeViewColumn(_(u"Subtitles"), renderer, text=0)
         column.set_resizable(True)
         column.set_expand(True)
         self.treeview.append_column(column)
 	# translators comment:
 	# This is the file-type of the subtitle file detected
-        column = Gtk.TreeViewColumn(_("Format"), renderer, text=1)
+        column = Gtk.TreeViewColumn(_(u"Format"), renderer, text=1)
         column.set_resizable(False)
         self.treeview.append_column(column)
 	# translators comment:
 	# This is a rating of the quality of the subtitle
-        column = Gtk.TreeViewColumn(_("Rating"), renderer, text=2)
+        column = Gtk.TreeViewColumn(_(u"Rating"), renderer, text=2)
         column.set_resizable(False)
         self.treeview.append_column(column)
 
@@ -403,9 +406,9 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.treeview.get_selection().connect('changed', self.on_treeview__row_change)
         self.treeview.connect('row-activated', self.on_treeview__row_activate)
 
-    def os_show_dialog(self, action, totem_object):
+    def os_show_dialog(self, action):
         if not self.dialog:
-            self.os_build_dialog(action, totem_object)
+            self.os_build_dialog(action)
 
         filename = self.totem.get_current_mrl()
         if not self.model.results or filename != self.filename:
@@ -422,8 +425,9 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.os_action_group = Gtk.ActionGroup(name='OpenSubtitles')
 
         self.action = Gtk.Action(name='opensubtitles',
-                                 label=_('_Download Movie Subtitles...'),
-                                 tooltip=_("Download movie subtitles from OpenSubtitles"))
+                                 label=_(u'_Download Movie Subtitles…'),
+                                 tooltip=_(u"Download movie subtitles from OpenSubtitles"),
+                                 stock_id=None)
 
         self.os_action_group.add_action(self.action)
 
@@ -441,7 +445,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.manager.ensure_update()
 
-        self.action.connect('activate', self.os_show_dialog, self.totem)
+        self.action.connect('activate', self.os_show_dialog)
 
         self.action.set_sensitive(self.totem.is_playing() and
 				  self.os_check_allowed_scheme() and
@@ -458,7 +462,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         # I think we must use video widget metadata but I don't found a way 
 	# to get this info from python
         filename = self.totem.get_current_mrl()
-        if Gio.content_type_guess(filename, '', 0)[0].split('/')[0] == 'audio':
+        if Gio.content_type_guess(filename, '')[0].split('/')[0] == 'audio':
             return True
         return False
 
@@ -475,13 +479,13 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.apply_button.set_sensitive(False)
 	self.find_button.set_sensitive(False)        
 
-        self.dialog.window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.dialog.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
 
         thread = SearchThread(self.model)
         thread.start()
         gobject.idle_add(self.os_populate_treeview)
 
-        self.progress.set_text(_('Searching subtitles...'))
+        self.progress.set_text(_(u'Searching subtitles…'))
         gobject.timeout_add(350, self.os_progress_bar_increment, thread)
 
     def os_populate_treeview(self):
@@ -502,14 +506,14 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.model.lock.release()
 
-        self.dialog.window.set_cursor(None)
+        self.dialog.get_window().set_cursor(None)
 
         return False
 
     def os_save_selected_subtitle(self, filename=None):
         """
         """
-        self.dialog.window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.dialog.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
 
         model, rows = self.treeview.get_selection().get_selected_rows()
         if rows:
@@ -539,7 +543,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
             thread.start()
             gobject.idle_add(self.os_save_subtitles, filename)
 
-            self.progress.set_text(_('Downloading the subtitles...'))
+            self.progress.set_text(_(u'Downloading the subtitles…'))
             gobject.timeout_add(350, self.os_progress_bar_increment, thread)
         else:
             #warn user!
@@ -556,7 +560,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 		if fp.query_exists(None):
                     fp.delete(None)
 
-            fp = Gio.file_new_for_path(filename)
+            fp = Gio.file_new_for_uri (filename)
             suburi = fp.get_uri ()
 
             subFile  = fp.replace('', False)
@@ -565,7 +569,8 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.model.lock.release()
 
-        self.dialog.window.set_cursor(None)
+        self.dialog.get_window().set_cursor(None)
+        self.on_close_clicked (None)
 
         if suburi:
             self.totem.set_current_subtitle(suburi)
@@ -599,7 +604,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
     # Callbacks
 
     def on_window__key_press_event(self, widget, event):
-        if event.keyval == Gdk.Escape:
+        if event.keyval == Gdk.KEY_Escape:
             self.dialog.destroy()
             self.dialog = None
             return True
@@ -644,8 +649,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
     def on_combobox__changed(self, combobox):
         iter = combobox.get_active_iter()
         self.model.lang = LANGUAGES[combobox.get_model().get_value(iter, 1)]
-        self.gconf_set_str(self.GCONF_BASE_DIR + self.GCONF_LANGUAGE,
-                           self.model.lang)
+        self.settings.set_string('language', self.model.lang)
 
     def on_close_clicked(self, data):
         self.dialog.destroy()
@@ -662,14 +666,3 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.os_get_results()
 
-    def gconf_get_str(self, key, default = ""):
-        val = self.gconf_client.get(key)
-        
-        if val is not None and val.type == gconf.VALUE_STRING:
-            return val.get_string()
-        else:
-            return default
-
-    def gconf_set_str(self, key, val):
-        self.gconf_client.set_string(key, val)
-
