@@ -28,7 +28,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gconf/gconf-client.h>
 #include <gio/gio.h>
 #include <string.h>
 
@@ -41,7 +40,10 @@
 #define PL_LEN (gtk_tree_model_iter_n_children (playlist->priv->model, NULL))
 
 static void ensure_shuffled (TotemPlaylist *playlist);
-static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
+static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist,
+					    const char    *mrl,
+					    const char    *display_name,
+					    const char    *content_type);
 
 typedef gboolean (*ClearComparisonFunc) (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data);
 
@@ -93,7 +95,8 @@ struct TotemPlaylistPrivate
 	int *shuffled;
 	int current_shuffled, shuffle_len;
 
-	GConfClient *gc;
+	GSettings *settings;
+	GSettings *lockdown_settings;
 
 	/* Used to know the position for drops */
 	GtkTreePath *tree_path;
@@ -143,6 +146,7 @@ enum {
 	TITLE_CUSTOM_COL,
 	SUBTITLE_URI_COL,
 	FILE_MONITOR_COL,
+	MIME_TYPE_COL,
 	NUM_COLS
 };
 
@@ -489,6 +493,8 @@ gtk_tree_selection_has_selected (GtkTreeSelection *selection)
 static void
 drop_finished_cb (TotemPlaylist *playlist, GAsyncResult *result, gpointer user_data)
 {
+	totem_playlist_add_mrls_finish (playlist, result, NULL);
+
 	/* Emit the "changed" signal once the last dropped MRL has been added to the playlist */
 	g_signal_emit (G_OBJECT (playlist),
 	               totem_playlist_table_signals[CHANGED], 0,
@@ -506,7 +512,7 @@ drop_cb (GtkWidget        *widget,
 	 TotemPlaylist    *playlist)
 {
 	char **list;
-	GList *p, *file_list;
+	GList *p, *file_list, *mrl_list = NULL;
 	guint i;
 	GdkDragAction action;
 
@@ -570,16 +576,16 @@ drop_cb (GtkWidget        *widget,
 			}
 		}
 
-		/* Add the MRL to the playlist asynchronously. If it's the last MRL, emit the "changed"
-		 * signal once we're done adding it */
-		if (p->next == NULL)
-			totem_playlist_add_mrl (playlist, filename, title, TRUE, NULL, (GAsyncReadyCallback) drop_finished_cb, NULL);
-		else
-			totem_playlist_add_mrl (playlist, filename, title, TRUE, NULL, NULL, NULL);
-
+		/* Add the MRL to the list of MRLs to be added to the playlist */
+		mrl_list = g_list_prepend (mrl_list, totem_playlist_mrl_data_new (filename, title));
 		g_free (filename);
 	}
 
+	/* Add all the MRLs to the playlist asynchronously, emitting the "changed" signal once we're done.
+	 * Note that this takes ownership of @mrl_list. */
+	if (mrl_list != NULL)
+		totem_playlist_add_mrls (playlist, g_list_reverse (mrl_list), TRUE, NULL, (GAsyncReadyCallback) drop_finished_cb, NULL);
+
 	g_strfreev (list);
 	g_list_free (file_list);
 	gtk_drag_finish (context, TRUE, FALSE, _time);
@@ -853,20 +859,22 @@ void
 totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	GSList *filenames, *l;
+	GList *mrl_list = NULL;
 
 	filenames = totem_add_files (totem_playlist_get_toplevel (playlist), NULL);
 	if (filenames == NULL)
 		return;
 
 	for (l = filenames; l != NULL; l = l->next) {
-		char *mrl;
-
-		mrl = l->data;
-		totem_playlist_add_mrl (playlist, mrl, NULL, TRUE, NULL, NULL, NULL);
+		char *mrl = l->data;
+		mrl_list = g_list_prepend (mrl_list, totem_playlist_mrl_data_new (mrl, NULL));
 		g_free (mrl);
 	}
 
 	g_slist_free (filenames);
+
+	if (mrl_list != NULL)
+		totem_playlist_add_mrls (playlist, g_list_reverse (mrl_list), TRUE, NULL, NULL, NULL);
 }
 
 static void
@@ -1209,7 +1217,7 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 	/* Special case some shortcuts */
 	if (event->state != 0) {
 		if ((event->state & GDK_CONTROL_MASK)
-		    && event->keyval == GDK_a) {
+		    && event->keyval == GDK_KEY_a) {
 			gtk_tree_selection_select_all
 				(playlist->priv->selection);
 			return TRUE;
@@ -1227,7 +1235,7 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 			|| (event->state & GDK_MOD5_MASK)))
 		return FALSE;
 
-	if (event->keyval == GDK_Delete)
+	if (event->keyval == GDK_KEY_Delete)
 	{
 		playlist_remove_files (playlist);
 		return TRUE;
@@ -1252,6 +1260,7 @@ set_playing_icon (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 		case TOTEM_PLAYLIST_STATUS_PAUSED:
 			icon_name = "media-playback-pause-symbolic";
 			break;
+		case TOTEM_PLAYLIST_STATUS_NONE:
 		default:
 			icon_name = NULL;
 	}
@@ -1421,20 +1430,16 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 }
 
 static void
-update_repeat_cb (GConfClient *client, guint cnxn_id,
-		GConfEntry *entry, TotemPlaylist *playlist)
+update_repeat_cb (GSettings *settings, const gchar *key, TotemPlaylist *playlist)
 {
-	gboolean repeat;
-
-	repeat = gconf_value_get_bool (entry->value);
-	playlist->priv->repeat = (repeat != FALSE);
+	playlist->priv->repeat = g_settings_get_boolean (settings, "repeat");
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[REPEAT_TOGGLED], 0,
-			repeat, NULL);
+			playlist->priv->repeat, NULL);
 }
 
 typedef struct {
@@ -1510,14 +1515,11 @@ ensure_shuffled (TotemPlaylist *playlist)
 }
 
 static void
-update_shuffle_cb (GConfClient *client, guint cnxn_id,
-		GConfEntry *entry, TotemPlaylist *playlist)
+update_shuffle_cb (GSettings *settings, const gchar *key, TotemPlaylist *playlist)
 {
-	gboolean shuffle;
+	playlist->priv->shuffle = g_settings_get_boolean (settings, "shuffle");
 
-	shuffle = gconf_value_get_bool (entry->value);
-	playlist->priv->shuffle = shuffle;
-	if (shuffle == FALSE) {
+	if (playlist->priv->shuffle == FALSE) {
 		g_free (playlist->priv->shuffled);
 		playlist->priv->shuffled = NULL;
 		playlist->priv->shuffle_len = 0;
@@ -1530,49 +1532,32 @@ update_shuffle_cb (GConfClient *client, guint cnxn_id,
 			NULL);
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[SHUFFLE_TOGGLED], 0,
-			shuffle, NULL);
+			playlist->priv->shuffle, NULL);
 }
 
 static void
-update_lockdown (GConfClient *client, guint cnxn_id,
-		GConfEntry *entry, TotemPlaylist *playlist)
+update_lockdown_cb (GSettings *settings, const gchar *key, TotemPlaylist *playlist)
 {
-	playlist->priv->disable_save_to_disk = gconf_client_get_bool
-			(playlist->priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
+	playlist->priv->disable_save_to_disk = g_settings_get_boolean (settings, "disable-save-to-disk");
 	totem_playlist_update_save_button (playlist);
 }
 
 static void
 init_config (TotemPlaylist *playlist)
 {
-	playlist->priv->gc = gconf_client_get_default ();
+	playlist->priv->settings = g_settings_new (TOTEM_GSETTINGS_SCHEMA);
+	playlist->priv->lockdown_settings = g_settings_new ("org.gnome.desktop.lockdown");
 
-	playlist->priv->disable_save_to_disk = gconf_client_get_bool
-	       		(playlist->priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
+	playlist->priv->disable_save_to_disk = g_settings_get_boolean (playlist->priv->lockdown_settings, "disable-save-to-disk");
 	totem_playlist_update_save_button (playlist);
 
-	gconf_client_add_dir (playlist->priv->gc, GCONF_PREFIX,
-			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	gconf_client_notify_add (playlist->priv->gc, GCONF_PREFIX"/repeat",
-			(GConfClientNotifyFunc) update_repeat_cb,
-			playlist, NULL, NULL);
-	gconf_client_notify_add (playlist->priv->gc, GCONF_PREFIX"/shuffle",
-			(GConfClientNotifyFunc) update_shuffle_cb,
-			playlist, NULL, NULL);
+	g_signal_connect (playlist->priv->lockdown_settings, "changed::disable-save-to-disk", (GCallback) update_lockdown_cb, playlist);
 
-	gconf_client_add_dir (playlist->priv->gc, "/desktop/gnome/lockdown",
-			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	gconf_client_notify_add (playlist->priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk",
-			(GConfClientNotifyFunc) update_lockdown,
-			playlist, NULL, NULL);
+	playlist->priv->repeat = g_settings_get_boolean (playlist->priv->settings, "repeat");
+	playlist->priv->shuffle = g_settings_get_boolean (playlist->priv->settings, "shuffle");
 
-	playlist->priv->repeat = gconf_client_get_bool (playlist->priv->gc,
-			GCONF_PREFIX"/repeat", NULL) != FALSE;
-	playlist->priv->shuffle = gconf_client_get_bool (playlist->priv->gc,
-			GCONF_PREFIX"/shuffle", NULL) != FALSE;
+	g_signal_connect (playlist->priv->settings, "changed::repeat", (GCallback) update_repeat_cb, playlist);
+	g_signal_connect (playlist->priv->settings, "changed::shuffle", (GCallback) update_shuffle_cb, playlist);
 }
 
 static void
@@ -1581,7 +1566,7 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 			     GHashTable *metadata,
 			     TotemPlaylist *playlist)
 {
-	const char *title;
+	const char *title, *content_type;
 	gint64 duration;
 
 	/* We ignore 0-length items in playlists, they're usually just banners */
@@ -1590,7 +1575,8 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	if (duration == 0)
 		return;
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
-	totem_playlist_add_one_mrl (playlist, uri, title);
+	content_type = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE);
+	totem_playlist_add_one_mrl (playlist, uri, title, content_type);
 }
 
 static gboolean
@@ -1647,6 +1633,14 @@ totem_playlist_dispose (GObject *object)
 		playlist->priv->action_group = NULL;
 	}
 
+	if (playlist->priv->settings != NULL)
+		g_object_unref (playlist->priv->settings);
+	playlist->priv->settings = NULL;
+
+	if (playlist->priv->lockdown_settings != NULL)
+		g_object_unref (playlist->priv->lockdown_settings);
+	playlist->priv->lockdown_settings = NULL;
+
 	G_OBJECT_CLASS (totem_playlist_parent_class)->dispose (object);
 }
 
@@ -1666,6 +1660,8 @@ totem_playlist_init (TotemPlaylist *playlist)
 {
 	GtkWidget *container;
 	GtkBuilder *xml;
+	GtkWidget *widget;
+	GtkStyleContext *context;
 
 	playlist->priv = G_TYPE_INSTANCE_GET_PRIVATE (playlist, TOTEM_TYPE_PLAYLIST, TotemPlaylistPrivate);
 	playlist->priv->parser = totem_pl_parser_new ();
@@ -1703,6 +1699,14 @@ totem_playlist_init (TotemPlaylist *playlist)
 	playlist->priv->up_button = GTK_WIDGET (gtk_builder_get_object (xml, "up_button"));
 	playlist->priv->down_button = GTK_WIDGET (gtk_builder_get_object (xml, "down_button"));
 
+	/* Join treeview and buttons */
+	widget = GTK_WIDGET (gtk_builder_get_object (xml, ("scrolledwindow1")));
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_set_junction_sides (context, GTK_JUNCTION_BOTTOM);
+	widget = GTK_WIDGET (gtk_builder_get_object (xml, ("toolbar1")));
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_set_junction_sides (context, GTK_JUNCTION_TOP);
+
 	/* Reparent the vbox */
 	container = GTK_WIDGET (gtk_builder_get_object (xml, "vbox4"));
 	g_object_ref (container);
@@ -1747,7 +1751,8 @@ totem_playlist_new (void)
 static gboolean
 totem_playlist_add_one_mrl (TotemPlaylist *playlist,
 			    const char *mrl,
-			    const char *display_name)
+			    const char *display_name,
+			    const char *content_type)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -1804,6 +1809,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist,
 			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
 			FILE_MONITOR_COL, monitor,
+			MIME_TYPE_COL, content_type,
 			-1);
 	g_free (escaped_filename);
 
@@ -1856,7 +1862,7 @@ static gboolean
 handle_parse_result (TotemPlParserResult res, TotemPlaylist *playlist, const gchar *mrl, const gchar *display_name)
 {
 	if (res == TOTEM_PL_PARSER_RESULT_UNHANDLED)
-		return totem_playlist_add_one_mrl (playlist, mrl, display_name);
+		return totem_playlist_add_one_mrl (playlist, mrl, display_name, NULL);
 	if (res == TOTEM_PL_PARSER_RESULT_ERROR) {
 		char *msg;
 
@@ -1945,6 +1951,257 @@ totem_playlist_add_mrl_sync (TotemPlaylist *playlist, const char *mrl, const cha
 	return handle_parse_result (totem_pl_parser_parse (playlist->priv->parser, mrl, FALSE), playlist, mrl, display_name);
 }
 
+typedef struct {
+	TotemPlaylist *playlist;
+	GList *mrls; /* list of TotemPlaylistMrlDatas */
+	gboolean cursor;
+	GAsyncReadyCallback callback;
+	gpointer user_data;
+
+	guint next_index_to_add;
+	GList *unadded_entries; /* list of TotemPlaylistMrlDatas */
+	volatile gint entries_remaining;
+} AddMrlsOperationData;
+
+static void
+add_mrls_operation_data_free (AddMrlsOperationData *data)
+{
+	/* Remove the cursor, if one was set */
+	if (data->cursor)
+		unset_waiting_cursor (data->playlist);
+
+	g_list_foreach (data->mrls, (GFunc) totem_playlist_mrl_data_free, NULL);
+	g_list_free (data->mrls);
+	g_object_unref (data->playlist);
+
+	g_slice_free (AddMrlsOperationData, data);
+}
+
+struct TotemPlaylistMrlData {
+	gchar *mrl;
+	gchar *display_name;
+	TotemPlParserResult res;
+
+	/* Implementation details */
+	AddMrlsOperationData *operation_data;
+	guint index;
+};
+
+/**
+ * totem_playlist_mrl_data_new:
+ * @mrl: a MRL
+ * @display_name: (allow-none): a human-readable display name for the MRL, or %NULL
+ *
+ * Create a new #TotemPlaylistMrlData struct storing the given @mrl and @display_name.
+ *
+ * This will typically be immediately appended to a #GList to be passed to totem_playlist_add_mrls().
+ *
+ * Return value: (transfer full): a new #TotemPlaylistMrlData; free with totem_playlist_mrl_data_free()
+ *
+ * Since: 3.0
+ */
+TotemPlaylistMrlData *
+totem_playlist_mrl_data_new (const gchar *mrl,
+                             const gchar *display_name)
+{
+	TotemPlaylistMrlData *data;
+
+	g_return_val_if_fail (mrl != NULL && *mrl != '\0', NULL);
+
+	data = g_slice_new (TotemPlaylistMrlData);
+	data->mrl = g_strdup (mrl);
+	data->display_name = g_strdup (display_name);
+
+	return data;
+}
+
+/**
+ * totem_playlist_mrl_data_free:
+ * @data: (transfer full): a #TotemPlaylistMrlData
+ *
+ * Free the given #TotemPlaylistMrlData struct. This should not generally be called by code outside #TotemPlaylist.
+ *
+ * Since: 3.0
+ */
+void
+totem_playlist_mrl_data_free (TotemPlaylistMrlData *data)
+{
+	g_return_if_fail (data != NULL);
+
+	/* NOTE: This doesn't call add_mrls_operation_data_free() on @data->operation_data, since it's shared with other instances of
+	 * TotemPlaylistMrlData, and not truly reference counted. */
+	g_free (data->display_name);
+	g_free (data->mrl);
+
+	g_slice_free (TotemPlaylistMrlData, data);
+}
+
+static void
+add_mrls_finish_operation (AddMrlsOperationData *operation_data)
+{
+	/* Check whether this is the final callback invocation; iff it is, we can call the user's callback for the entire operation and free the
+	 * operation data */
+	if (g_atomic_int_dec_and_test (&(operation_data->entries_remaining)) == TRUE) {
+		GSimpleAsyncResult *async_result;
+
+		async_result = g_simple_async_result_new (G_OBJECT (operation_data->playlist), operation_data->callback, operation_data->user_data,
+		                                          totem_playlist_add_mrls);
+		g_simple_async_result_complete (async_result);
+		g_object_unref (async_result);
+
+		add_mrls_operation_data_free (operation_data);
+	}
+}
+
+/* Called exactly once for each MRL in a totem_playlist_add_mrls() operation. Called in the thread running the main loop. If the MRL which has just
+ * been parsed is the next one in the sequence (of entries in @mrls as passed to totem_playlist_add_mrls()), it's added to the playlist proper.
+ * Otherwise, it's added to a sorted queue of MRLs which have had their callbacks called out of order.
+ * When a MRL is added to the playlist proper, any successor MRLs which are in the sorted queue are also added to the playlist proper.
+ * When add_mrls_cb() is called for the last time for a given call to totem_playlist_add_mrls(), it calls the user's callback for the operation
+ * (passed as @callback to totem_playlist_add_mrls()) and frees the #AddMrlsOperationData struct. This is handled by add_mrls_finish_operation().
+ * The #TotemPlaylistMrlData for each MRL is freed by add_mrls_operation_data_free() at the end of the entire operation. */
+static void
+add_mrls_cb (TotemPlParser *parser, GAsyncResult *result, TotemPlaylistMrlData *mrl_data)
+{
+	AddMrlsOperationData *operation_data = mrl_data->operation_data;
+
+	/* Finish parsing the playlist */
+	mrl_data->res = totem_pl_parser_parse_finish (parser, result, NULL);
+
+	g_assert (mrl_data->index >= operation_data->next_index_to_add);
+
+	if (mrl_data->index == operation_data->next_index_to_add) {
+		GList *i;
+
+		/* The entry is the next one in the order, so doesn't need to be added to the unadded list, and can be added to playlist proper */
+		operation_data->next_index_to_add++;
+		handle_parse_result (mrl_data->res, operation_data->playlist, mrl_data->mrl, mrl_data->display_name);
+
+		/* See if we can now add any other entries which have already been processed */
+		for (i = operation_data->unadded_entries;
+		     i != NULL && ((TotemPlaylistMrlData*) i->data)->index == operation_data->next_index_to_add;
+		     i = g_list_delete_link (i, i)) {
+			TotemPlaylistMrlData *_mrl_data = (TotemPlaylistMrlData*) i->data;
+
+			operation_data->next_index_to_add++;
+			handle_parse_result (_mrl_data->res, operation_data->playlist, _mrl_data->mrl, _mrl_data->display_name);
+		}
+
+		operation_data->unadded_entries = i;
+	} else {
+		GList *i;
+
+		/* The entry has been parsed out of order, so needs to be added (in the correct position) to the unadded list for latter addition to
+		 * the playlist proper */
+		for (i = operation_data->unadded_entries; i != NULL && mrl_data->index > ((TotemPlaylistMrlData*) i->data)->index; i = i->next);
+		operation_data->unadded_entries = g_list_insert_before (operation_data->unadded_entries, i, mrl_data);
+	}
+
+	/* Check whether this is the last callback; call the user's callback for the entire operation and free the operation data if appropriate */
+	add_mrls_finish_operation (operation_data);
+}
+
+/**
+ * totem_playlist_add_mrls:
+ * @self: a #TotemPlaylist
+ * @mrls: (element-type TotemPlaylistMrlData) (transfer full): a list of #TotemPlaylistMrlData structs
+ * @cursor: %TRUE to set a waiting cursor on the playlist for the duration of the operation, %FALSE otherwise
+ * @cancellable: (allow-none): a #Cancellable, or %NULL
+ * @callback: (scope async) (allow-none): callback to call once all the MRLs have been added to the playlist, or %NULL
+ * @user_data: (closure) (allow-none): user data to pass to @callback, or %NULL
+ *
+ * Add the MRLs listed in @mrls to the playlist asynchronously, and ensuring that they're added to the playlist in the order they appear in the
+ * input #GList.
+ *
+ * @mrls should be a #GList of #TotemPlaylistMrlData structs, each created with totem_playlist_mrl_data_new(). This function takes ownership of both
+ * the list and its elements when called, so don't free either after calling totem_playlist_add_mrls().
+ *
+ * @callback will be called after all the MRLs in @mrls have been parsed and (if they were parsed successfully) added to the playlist. In the
+ * callback function, totem_playlist_add_mrls_finish() should be called to check for errors.
+ *
+ * Since: 3.0
+ */
+void
+totem_playlist_add_mrls (TotemPlaylist *self,
+                         GList *mrls,
+                         gboolean cursor,
+                         GCancellable *cancellable,
+                         GAsyncReadyCallback callback,
+                         gpointer user_data)
+{
+	AddMrlsOperationData *operation_data;
+	GList *i;
+	guint mrl_index = 0;
+
+	g_return_if_fail (TOTEM_IS_PLAYLIST (self));
+	g_return_if_fail (mrls != NULL);
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	/* Build the data struct to pass to the callback function */
+	operation_data = g_slice_new (AddMrlsOperationData);
+	operation_data->playlist = g_object_ref (self);
+	operation_data->mrls = mrls;
+	operation_data->cursor = cursor;
+	operation_data->callback = callback;
+	operation_data->user_data = user_data;
+	operation_data->next_index_to_add = mrl_index;
+	operation_data->unadded_entries = NULL;
+	g_atomic_int_set (&(operation_data->entries_remaining), 1);
+
+	/* Display a waiting cursor if required */
+	if (cursor)
+		set_waiting_cursor (self);
+
+	for (i = mrls; i != NULL; i = i->next) {
+		TotemPlaylistMrlData *mrl_data = (TotemPlaylistMrlData*) i->data;
+
+		if (mrl_data == NULL)
+			continue;
+
+		/* Set the item's parsing index, so that it's inserted into the playlist in the position it appeared in @mrls */
+		mrl_data->operation_data = operation_data;
+		mrl_data->index = mrl_index++;
+
+		g_atomic_int_inc (&(operation_data->entries_remaining));
+
+		/* Start parsing the playlist. Once this is complete, add_mrls_cb() is called (i.e. it's called exactly once for each entry in
+		 * @mrls).
+		 * TODO: Cancellation is currently not supoprted, since no consumers of this API make use of it, and it needs careful thought when
+		 * being implemented, as a separate #GCancellable instance will have to be created for each parallel computation. */
+		totem_pl_parser_parse_async (self->priv->parser, mrl_data->mrl, FALSE, NULL, (GAsyncReadyCallback) add_mrls_cb, mrl_data);
+	}
+
+	/* Deal with the case that all async operations completed before we got to this point (since we've held a reference to the operation data so
+	 * that it doesn't get freed prematurely if all the scheduled async parse operations complete before we've finished scheduling the rest. */
+	add_mrls_finish_operation (operation_data);
+}
+
+/**
+ * totem_playlist_add_mrls_finish:
+ * @self: a #TotemPlaylist
+ * @result: the #GAsyncResult that was provided to the callback
+ * @error: (allow-none): a #GError for error reporting, or %NULL
+ *
+ * Finish an asynchronous batch MRL addition operation started by totem_playlist_add_mrls().
+ *
+ * Return value: %TRUE on success, %FALSE otherwise
+ *
+ * Since: 3.0
+ */
+gboolean
+totem_playlist_add_mrls_finish (TotemPlaylist *self,
+                                GAsyncResult *result,
+                                GError **error)
+{
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (self), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self), totem_playlist_add_mrls), FALSE);
+
+	/* We don't have anything to return at the moment. */
+	return TRUE;
+}
+
 static gboolean
 totem_playlist_clear_cb (GtkTreeModel *model,
 			 GtkTreePath  *path,
@@ -2220,7 +2477,7 @@ totem_playlist_get_current_mrl (TotemPlaylist *playlist, char **subtitle)
 }
 
 char *
-totem_playlist_get_current_title (TotemPlaylist *playlist, gboolean *custom)
+totem_playlist_get_current_title (TotemPlaylist *playlist, char **content_type)
 {
 	GtkTreeIter iter;
 	char *path;
@@ -2234,11 +2491,11 @@ totem_playlist_get_current_title (TotemPlaylist *playlist, gboolean *custom)
 			&iter,
 			playlist->priv->current);
 
-	if (custom != NULL) {
+	if (content_type != NULL) {
 		gtk_tree_model_get (playlist->priv->model,
 				    &iter,
 				    FILENAME_COL, &path,
-				    TITLE_CUSTOM_COL, custom,
+				    MIME_TYPE_COL, content_type,
 				    -1);
 	} else {
 		gtk_tree_model_get (playlist->priv->model,
@@ -2332,11 +2589,10 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 }
 
 gboolean
-totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean force)
+totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
-	gboolean custom_title;
 	char *escaped_title;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
@@ -2349,17 +2605,6 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 			&iter,
 			playlist->priv->current);
 
-	if (&iter == NULL)
-		return FALSE;
-
-	if (force == FALSE) {
-		gtk_tree_model_get (playlist->priv->model, &iter,
-				TITLE_CUSTOM_COL, &custom_title,
-				-1);
-		if (custom_title != FALSE)
-			return TRUE;
-	}
-
 	escaped_title = g_markup_escape_text (title, -1);
 	gtk_list_store_set (store, &iter,
 			FILENAME_COL, title,
@@ -2391,8 +2636,6 @@ totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state)
 			&iter,
 			playlist->priv->current);
 
-	g_return_val_if_fail (&iter != NULL, FALSE);
-
 	gtk_list_store_set (store, &iter,
 			PLAYING_COL, state,
 			-1);
@@ -2526,8 +2769,7 @@ totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 {
 	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
-	gconf_client_set_bool (playlist->priv->gc, GCONF_PREFIX"/repeat",
-			repeat, NULL);
+	g_settings_set_boolean (playlist->priv->settings, "repeat", repeat);
 }
 
 gboolean
@@ -2543,8 +2785,7 @@ totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 {
 	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
-	gconf_client_set_bool (playlist->priv->gc, GCONF_PREFIX"/shuffle",
-			shuffle, NULL);
+	g_settings_set_boolean (playlist->priv->settings, "shuffle", shuffle);
 }
 
 void
