@@ -35,6 +35,8 @@
  *
  * #TotemVideoList is a #GtkTreeView designed for listing videos by their thumbnails using #TotemCellRendererVideo.
  * It supports tooltips, loading the videos by activating tree view rows, and #GtkUIManager actions in the popup menu.
+ *
+ * Note that it requires #TotemVideoList:totem to be set to an instance of #TotemObject to work.
  **/
 
 #include "config.h"
@@ -56,6 +58,7 @@ struct _TotemVideoListPrivate {
 	GtkBuilder *xml;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
+	gboolean show_tooltip_uri;
 };
 
 #define TOTEM_VIDEO_LIST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TOTEM_TYPE_VIDEO_LIST, TotemVideoListPrivate))
@@ -63,7 +66,8 @@ struct _TotemVideoListPrivate {
 enum {
 	PROP_TOOLTIP_COLUMN = 1,
 	PROP_MRL_COLUMN,
-	PROP_TOTEM
+	PROP_TOTEM,
+	PROP_SHOW_TOOLTIP_URI
 };
 
 enum {
@@ -153,6 +157,18 @@ totem_video_list_class_init (TotemVideoListClass *klass)
 				g_param_spec_object ("totem", NULL, NULL,
 					TOTEM_TYPE_OBJECT, G_PARAM_READWRITE));
 
+	/**
+	 * TotemVideoList:show-tooltip-uri:
+	 *
+	 * If this is %TRUE, the URI of each video will be displayed in the tooltip for that video (or a message saying "No video URI" will be
+	 * displayed if the URI is unset); otherwise, the tooltip will only display the video name.
+	 *
+	 * Since: 2.90.6
+	 **/
+	g_object_class_install_property (object_class, PROP_SHOW_TOOLTIP_URI,
+				g_param_spec_boolean ("show-tooltip-uri", NULL, NULL,
+					TRUE, G_PARAM_READWRITE));
+
 	/**
 	 * TotemVideoList::starting-video:
 	 * @video_list: the #TotemVideoList which received the signal
@@ -182,6 +198,7 @@ totem_video_list_init (TotemVideoList *self)
 	self->priv->totem = NULL;
 	self->priv->tooltip_column = -1;
 	self->priv->mrl_column = -1;
+	self->priv->show_tooltip_uri = TRUE;
 
 	/* Get the interface */
 	self->priv->xml = totem_interface_load ("video-list.ui", TRUE, NULL, self);
@@ -244,6 +261,10 @@ totem_video_list_set_property (GObject *object, guint property_id, const GValue
 			priv->totem = (Totem*) g_value_dup_object (value);
 			g_object_notify (object, "totem");
 			break;
+		case PROP_SHOW_TOOLTIP_URI:
+			priv->show_tooltip_uri = g_value_get_boolean (value);
+			g_object_notify (object, "show-tooltip-uri");
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
@@ -265,6 +286,9 @@ totem_video_list_get_property (GObject *object, guint property_id, GValue *value
 		case PROP_TOTEM:
 			g_value_set_object (value, G_OBJECT (priv->totem));
 			break;
+		case PROP_SHOW_TOOLTIP_URI:
+			g_value_set_boolean (value, priv->show_tooltip_uri);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
@@ -290,7 +314,7 @@ query_tooltip_cb (GtkWidget *widget, gint x, gint y, gboolean keyboard_mode, Gtk
 				&model, &path, &iter))
 		return FALSE;
 
-	if (self->priv->mrl_column == -1) {
+	if (self->priv->mrl_column == -1 || self->priv->show_tooltip_uri == FALSE) {
 		gtk_tree_model_get (model, &iter, self->priv->tooltip_column, &tooltip_text, -1);
 		gtk_tooltip_set_text (tooltip, tooltip_text);
 		g_free (tooltip_text);
@@ -328,7 +352,7 @@ static void
 row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
 {
 	GtkTreeIter iter;
-	gchar *mrl, *display_name;
+	gchar *mrl, *display_name = NULL;
 	gboolean play_video = TRUE;
 	TotemVideoList *self = TOTEM_VIDEO_LIST (tree_view);
 	GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
@@ -461,6 +485,7 @@ add_to_playlist_action_callback (GtkAction *action, TotemVideoList *self)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	gchar *mrl, *display_name;
+	GList *mrl_list = NULL;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self));
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (self));
 	GList *l = gtk_tree_selection_get_selected_rows (selection, NULL);
@@ -482,7 +507,7 @@ add_to_playlist_action_callback (GtkAction *action, TotemVideoList *self)
 			continue;
 		}
 
-		totem_playlist_add_mrl_with_cursor (playlist, mrl, display_name);
+		mrl_list = g_list_prepend (mrl_list, totem_playlist_mrl_data_new (mrl, display_name));
 
 		g_free (mrl);
 		g_free (display_name);
@@ -490,6 +515,10 @@ add_to_playlist_action_callback (GtkAction *action, TotemVideoList *self)
 
 	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (l);
+
+	/* Asynchronously add all the MRLs to the playlist in order */
+	if (mrl_list != NULL)
+		totem_playlist_add_mrls (playlist, g_list_reverse (mrl_list), TRUE, NULL, NULL, NULL);
 }
 
 void
@@ -536,7 +565,7 @@ error:
  *
  * Returns the #GtkUIManager in use by @self.
  *
- * Return value: @self's #GtkUIManager
+ * Return value: (transfer none): @self's #GtkUIManager
  **/
 GtkUIManager *
 totem_video_list_get_ui_manager (TotemVideoList *self)
